@@ -337,10 +337,7 @@ impanel_trigger_property_callback (GDBusConnection *connection,
         ibus_panel_impanel_exec_dialog((IBusPanelService *)user_data);
     else if (g_ascii_strncasecmp (prop_key, "Engine/", 7) == 0) {
         prop_key += 7;// +7 to skip "Engine/"
-        if (g_ascii_strncasecmp (prop_key, "None", 4) == 0)
-            ibus_input_context_disable(((IBusPanelImpanel *)user_data)->input_context);
-        else
-            ibus_input_context_set_engine(((IBusPanelImpanel *)user_data)->input_context, prop_key);
+        ibus_input_context_set_engine(((IBusPanelImpanel *)user_data)->input_context, prop_key);
     }
     else
 #if !IBUS_CHECK_VERSION(1,3,99)
@@ -575,36 +572,27 @@ ibus_panel_impanel_focus_in (IBusPanelService *panel,
     g_object_get (IBUS_PANEL_IMPANEL (panel),
                   "connection", &ibusconn,
                   NULL);
+    
+    IBusEngineDesc *engine_desc = ibus_bus_get_global_engine(IBUS_PANEL_IMPANEL (panel)->bus);
+    
     IBusInputContext *ic = ibus_input_context_get_input_context(input_context_path, ibusconn);
     IBUS_PANEL_IMPANEL (panel)->input_context = ic;
-
-    gboolean enable = ibus_input_context_is_enabled(ic);
-
-//     fprintf(stderr, "enable %d %s\n", enable, input_context_path);
-
-    if (enable == 0) {
-        ibus_property_set_icon (IBUS_PANEL_IMPANEL (panel)->logo_prop, "ibus");
-    }
-    else {
-        IBusEngineDesc *engine_desc = ibus_input_context_get_engine(ic);
-        if (!engine_desc) {
+    
+    const gchar* icon_name = "ibus-keyboard";
+    if (engine_desc) {
 #if !IBUS_CHECK_VERSION(1,3,99)
-//             if (error) {
-//                 *error = ibus_error_new_from_text (DBUS_ERROR_FAILED, "engine description is null");
-//             }
-            return FALSE;
+        icon_name = engine_desc->icon;
 #else
-            return;
-#endif
-        }
-
-#if !IBUS_CHECK_VERSION(1,3,99)
-        ibus_property_set_icon (IBUS_PANEL_IMPANEL (panel)->logo_prop, engine_desc->icon);
-#else
-        ibus_property_set_icon (IBUS_PANEL_IMPANEL (panel)->logo_prop, ibus_engine_desc_get_icon (engine_desc));
+        icon_name = ibus_engine_desc_get_icon (engine_desc);
 #endif
     }
 
+#if !IBUS_CHECK_VERSION(1,3,99)
+    ibus_property_set_icon (IBUS_PANEL_IMPANEL (panel)->logo_prop, icon_name);
+#else
+    ibus_property_set_icon (IBUS_PANEL_IMPANEL (panel)->logo_prop, icon_name);
+#endif
+
     char propstr[512];
     propstr[0] = '\0';
 
@@ -1174,28 +1162,21 @@ static void
 ibus_panel_impanel_state_changed (IBusPanelService *panel)
 #endif
 {
-    gboolean enable = ibus_input_context_is_enabled(IBUS_PANEL_IMPANEL (panel)->input_context);
-
-    if (enable == 0) {
-        ibus_property_set_icon (IBUS_PANEL_IMPANEL (panel)->logo_prop, "ibus");
-    }
-    else {
-        IBusEngineDesc *engine_desc = ibus_input_context_get_engine(IBUS_PANEL_IMPANEL (panel)->input_context);
-        if (!engine_desc) {
+    IBusEngineDesc *engine_desc = ibus_input_context_get_engine(IBUS_PANEL_IMPANEL (panel)->input_context);
+    if (!engine_desc) {
 #if !IBUS_CHECK_VERSION(1,3,99)
-            return FALSE;
+        return FALSE;
 #else
-            return;
-#endif
-        }
-
-#if !IBUS_CHECK_VERSION(1,3,99)
-        ibus_property_set_icon (IBUS_PANEL_IMPANEL (panel)->logo_prop, engine_desc->icon);
-#else
-        ibus_property_set_icon (IBUS_PANEL_IMPANEL (panel)->logo_prop, ibus_engine_desc_get_icon (engine_desc));
+        return;
 #endif
     }
 
+#if !IBUS_CHECK_VERSION(1,3,99)
+    ibus_property_set_icon (IBUS_PANEL_IMPANEL (panel)->logo_prop, engine_desc->icon);
+#else
+    ibus_property_set_icon (IBUS_PANEL_IMPANEL (panel)->logo_prop, ibus_engine_desc_get_icon (engine_desc));
+#endif
+
     char propstr[512];
     propstr[0] = '\0';
 
@@ -1208,7 +1189,7 @@ ibus_panel_impanel_state_changed (IBusPanelService *panel)
 
     g_dbus_connection_emit_signal (IBUS_PANEL_IMPANEL (panel)->conn,
                                    NULL, "/kimpanel", "org.kde.kimpanel.inputmethod", "Enable",
-                                   g_variant_new ("(b)", enable),
+                                   g_variant_new ("(b)", TRUE),
                                    NULL);
 #if !IBUS_CHECK_VERSION(1,3,99)
     _UNUSED(error);
@@ -1249,15 +1230,6 @@ ibus_panel_impanel_exec_menu (IBusPanelService *panel)
         g_variant_builder_add (&builder, "s", propstr);
     }
 
-    ibus_engine_desc_args_to_propstr("None",
-                                     NULL,
-                                     "Disable",
-                                     "ibus",
-                                     "",
-                                     propstr);
-
-    g_variant_builder_add (&builder, "s", propstr);
-
     g_dbus_connection_emit_signal (IBUS_PANEL_IMPANEL (panel)->conn,
                                    NULL, "/kimpanel", "org.kde.kimpanel.inputmethod", "ExecMenu",
                                    g_variant_new ("(as)", &builder),
