@@ -20,6 +20,7 @@
 #include "PanelIcon.h"
 #include "Layout.h"
 #include <QAction>
+#include <QFile>
 #include <QGraphicsView>
 
 #include <KConfigDialog>
@@ -32,8 +33,11 @@
 #include <Plasma/ToolTipContent>
 
 
-PanelIcon::PanelIcon(QObject *parent, const QVariantList &args)  :
-        Plasma::PopupApplet(parent, args), m_plasmaboard(0){
+PanelIcon::PanelIcon(QObject *parent, const QVariantList &args)
+    : Plasma::PopupApplet(parent, args),
+      m_plasmaboard(0),
+      m_tempLayout(false)
+{
     setAspectRatioMode(Plasma::IgnoreAspectRatio);
     setPopupIcon("preferences-desktop-keyboard");
     //setFocusPolicy(Qt::NoFocus);
@@ -43,8 +47,9 @@ PanelIcon::PanelIcon(QObject *parent, const QVariantList &args)  :
 }
 
 
-PanelIcon::~PanelIcon() {
-	Plasma::ToolTipManager::self()->unregisterWidget(this);
+PanelIcon::~PanelIcon()
+{
+    Plasma::ToolTipManager::self()->unregisterWidget(this);
     qDeleteAll(m_layouts);
 }
 
@@ -53,100 +58,68 @@ void PanelIcon::configAccepted()
     initKeyboard(m_layout);
 }
 
-void PanelIcon::configChanged(QString name)
+void PanelIcon::configChanged()
 {
-    Layout *lay = m_layouts[0];
-
-    Q_FOREACH(Layout* l, m_layouts){
-        if(l->name() == name){
-            lay = l;
-            break;
+    QString layout = config().readEntry("layout", QString());
+    if (layout.isEmpty()) {
+        // fallback to default layout
+        layout = KStandardDirs::locate("data", "plasmaboard/full.xml");
+    } else {
+        // lookup whether saved layout exists
+        const QString file = KStandardDirs::locate("data", layout); 
+        if (!file.isEmpty()) {
+            layout = file;
         }
     }
 
-    m_layout = lay->path();
-    ui.descriptionLabel->setText(lay->description());
+    // only rebuild the keyboard if the layout has actually changed
+    if (layout != m_layout && QFile::exists(layout)) {
+        m_layout = layout;
+        initKeyboard(m_layout);
+    }
 }
 
-/*QList<QAction*> PanelIcon::contextualActions(){
-    QList<QAction*> list;
-    Q_FOREACH(Layout* l, m_layouts){
-        QAction *action = new QAction(l->name(), this);
-        action->setData(l->path());
-        connect(action, SIGNAL(triggered(bool)), this, SLOT(initKeyboard()));
-        list << action;
+void PanelIcon::constraintsEvent(Plasma::Constraints constraints)
+{
+    if (constraints & Plasma::FormFactorConstraint) {
+        if (formFactor() == Plasma::Horizontal || formFactor() == Plasma::Vertical) {
+            Plasma::ToolTipManager::self()->registerWidget(this);
+            Plasma::ToolTipContent toolTip;
+            toolTip.setImage(KIcon("preferences-desktop-keyboard"));
+            toolTip.setMainText(i18n("Virtual Keyboard"));
+            Plasma::ToolTipManager::self()->setContent(this, toolTip);
+        } else {
+            Plasma::ToolTipManager::self()->unregisterWidget(this);
+        }
     }
-
-    QAction *sep = new QAction(this);
-    sep->setSeparator(true);
-    list << sep;
-
-    return list;
-}*/
+}
 
 void PanelIcon::createConfigurationInterface(KConfigDialog *parent)
 {
+    qDeleteAll(m_layouts);
+    m_layouts.clear();
+    QStringList layoutList = KGlobal::dirs()->findAllResources("data", "plasmaboard/*.xml");
+    foreach (QString path, layoutList) {
+        m_layouts << new Layout(path);
+    }
+
     QWidget *widget = new QWidget(parent);
     ui.setupUi(widget);
-    parent->addPage(widget, i18nc("Different keyboard layouts","Layouts"), "plasmaboard");
+    parent->addPage(widget, i18nc("Different keyboard layouts", "Layouts"), "plasmaboard");
     connect(parent, SIGNAL(applyClicked()), this, SLOT(configAccepted()));
     connect(parent, SIGNAL(okClicked()), this, SLOT(configAccepted()));
 
-    Q_FOREACH(Layout* l, m_layouts){
+
+    foreach (Layout * l, m_layouts) {
         ui.layoutsComboBox->addItem(l->name(), l->path());
-        if(l->path() == m_layout){
+        if (l->path() == m_layout) {
             ui.descriptionLabel->setText(l->description());
             ui.layoutsComboBox->setCurrentIndex(ui.layoutsComboBox->count() - 1);
         }
     }
 
-    connect(ui.layoutsComboBox, SIGNAL(currentIndexChanged(QString)), this, SLOT(configChanged(QString)));
-}
-
-
-void PanelIcon::init() {
-
-    QStringList layoutList = KGlobal::dirs()->findAllResources("data", "plasmaboard/*.xml");
-    Q_FOREACH(QString path, layoutList){
-        m_layouts << new Layout(path);
-    }
-
-    Plasma::ToolTipManager::self()->registerWidget(this);
-    Plasma::ToolTipContent toolTip;
-    toolTip.setImage(KIcon("preferences-desktop-keyboard"));
-    toolTip.setMainText(i18n("Virtual Keyboard"));
-    Plasma::ToolTipManager::self()->setContent(this, toolTip);
-
-
-    KConfigGroup cg = config();
-    QString layout;
-    layout = cg.readEntry("layout", layout);
-
-    QString file = KStandardDirs::locate("data", layout); // lookup whether saved layout exists
-    if(layout.size() > 0 && file.size() > 0){
-        m_layout = file;
-    }
-    else{ // fallback to default layout
-        m_layout = KStandardDirs::locate("data", "plasmaboard/full.xml");
-    }
-
-}
-
-void PanelIcon::initKeyboard() {
-    QString path = ((QAction*)sender())->data().toString();
-    m_plasmaboard->deleteKeys();
-    m_plasmaboard->initKeyboard(path);
-    m_plasmaboard->refreshKeys();
-    m_plasmaboard->update();
-    saveLayout(path);
-}
-
-void PanelIcon::initKeyboard(QString layoutFile) {
-    m_plasmaboard->deleteKeys();
-    m_plasmaboard->initKeyboard(layoutFile);
-    m_plasmaboard->refreshKeys();
-    m_plasmaboard->update();
-    saveLayout(layoutFile);
+    connect(ui.layoutsComboBox, SIGNAL(currentIndexChanged(QString)), this, SLOT(layoutNameChanged(QString)));
+    connect(ui.layoutsComboBox, SIGNAL(currentIndexChanged(int)), parent, SLOT(settingsModified()));
 }
 
 QGraphicsWidget *PanelIcon::graphicsWidget()
@@ -165,22 +138,104 @@ QGraphicsWidget *PanelIcon::graphicsWidget()
     return m_plasmaboard;
 }
 
-void PanelIcon::popupEvent(bool show){
-	if ( !show ) {
-        m_plasmaboard->reset();
-	}
+void PanelIcon::layoutNameChanged(const QString &name)
+{
+    Layout *lay = m_layouts[0];
+
+    foreach (Layout * l, m_layouts) {
+        if (l->name() == name) {
+            lay = l;
+            break;
+        }
+    }
+
+    m_layout = lay->path();
+    ui.descriptionLabel->setText(lay->description());
 }
 
-void PanelIcon::saveLayout(QString path) {
+void PanelIcon::init()
+{
+    configChanged();
+}
 
+void PanelIcon::initKeyboard()
+{
+    if (!m_plasmaboard) {
+        return;
+    }
+
+    QAction *action = qobject_cast<QAction *>(sender());
+    if (!action) {
+        return;
+    }
+
+    QString path = action->data().toString();
+    setLayout(path);
+    saveLayout(path);
+}
+
+void PanelIcon::initKeyboard(const QString &layoutFile)
+{
+    if (!m_plasmaboard) {
+        return;
+    }
+
+    setLayout(layoutFile);
+    saveLayout(layoutFile);
+}
+
+void PanelIcon::resetLayout()
+{
+    if (m_tempLayout) {
+        setLayout(m_layout);
+    }
+}
+
+void PanelIcon::showLayout(const QString &layout)
+{
+    kDebug() << layout;
+    if (layout.isEmpty()) {
+        resetLayout();
+        return;
+    }
+
+    const QString file = KStandardDirs::locate("data", "plasmaboard/" + layout);
+    if (!file.isEmpty()) {
+        setLayout(file);
+    } else if (QFile::exists(layout)) {
+        initKeyboard(layout);
+    }
+}
+
+void PanelIcon::setLayout(const QString &layoutFile)
+{
+    m_tempLayout = layoutFile != m_layout;
+    m_plasmaboard->deleteKeys();
+    m_plasmaboard->initKeyboard(layoutFile);
+    m_plasmaboard->refreshKeys();
+    m_plasmaboard->update();
+}
+
+void PanelIcon::popupEvent(bool show)
+{
+    if (!show) {
+        m_plasmaboard->reset();
+        resetLayout();
+    }
+}
+
+void PanelIcon::saveLayout(const QString &path)
+{
     int pos = path.indexOf("plasmaboard");
 
     KConfigGroup cg = config();
     cg.writeEntry("layout", path.right(path.size() - pos));
 
     emit configNeedsSaving();
-
 }
 
+#include "PanelIcon.moc"
+
 // This is the command that links your applet to the .desktop file
 K_EXPORT_PLASMA_APPLET(plasmaboard, PanelIcon)
+
