@@ -26,7 +26,6 @@
 #include <QRegExp>
 #include <KComboBox>
 #include <KLineEdit>
-#include <Plasma/ComboBox>
 #include <Plasma/LineEdit>
 #include <Plasma/Label>
 #include <Plasma/Frame>
@@ -34,12 +33,23 @@
 
 using namespace KUnitConversion;
 
+ComboBox::ComboBox(QGraphicsWidget* parent)
+    : Plasma::ComboBox(parent)
+{
+}
+
+void ComboBox::mousePressEvent(QGraphicsSceneMouseEvent* event)
+{
+    emit mousePressed();
+    Plasma::ComboBox::mousePressEvent(event);
+}
+
 Q_DECLARE_METATYPE(UnitPtr)
 Q_DECLARE_METATYPE(UnitCategory*)
 
 UnitConverter::UnitConverter(QObject *parent, const QVariantList &args)
 : Plasma::PopupApplet(parent, args)
-, m_widget(0)
+, m_widget(0), m_bCalculateReverse(false)
 {
     KGlobal::locale()->insertCatalog("libconversion");
     setAspectRatioMode(Plasma::IgnoreAspectRatio);
@@ -95,15 +105,29 @@ void UnitConverter::sltCategoryChanged(int index)
 void UnitConverter::sltUnitChanged(int index)
 {
     Q_UNUSED(index);
-    calculate();
+    if ( m_bCalculateReverse ) {
+        calculateReverse();
+    }
+    else {
+        calculate();
+    }
 }
 
 void UnitConverter::sltValueChanged(const QString &sNewValue)
 {
     Q_UNUSED(sNewValue);
+    m_bCalculateReverse = false; // store calculation direction
     calculate();
 }
 
+void UnitConverter::sltValueChangedReverse(const QString &sNewValue)
+{
+    Q_UNUSED(sNewValue);
+    m_bCalculateReverse = true; // store calculation direction
+    calculateReverse();
+}
+
+/// Calculates from left to right
 void UnitConverter::calculate()
 {
     UnitPtr in = m_pCmbUnit1->nativeWidget()->itemData(
@@ -119,7 +143,28 @@ void UnitConverter::calculate()
            QString addZero = QString::number(dblValueOut.number()) + ".0";
            m_pTxtValue2->setText(addZero);
         } else {
-           m_pTxtValue2->setText(QString::number(dblValueOut.number()));
+           m_pTxtValue2->setText(QString::number(dblValueOut.number(), 'g', 15));
+        }
+    }
+}
+
+/// Calculates from right to left
+void UnitConverter::calculateReverse()
+{
+    UnitPtr in = m_pCmbUnit2->nativeWidget()->itemData(
+            m_pCmbUnit2->nativeWidget()->currentIndex()).value<UnitPtr>();
+    UnitPtr out = m_pCmbUnit1->nativeWidget()->itemData(
+            m_pCmbUnit1->nativeWidget()->currentIndex()).value<UnitPtr>();
+    if (!in.isNull() && !out.isNull()) {
+        Value dblValueIn(m_pTxtValue2->text().toDouble(), in);
+        Value dblValueOut = dblValueIn.convertTo(out->id());
+        QRegExp decimalCheck("^\\d+\\.0$");
+        QRegExp onlyDecimal("^\\d+$");
+        if(decimalCheck.exactMatch(m_pTxtValue2->text()) && onlyDecimal.exactMatch(QString::number(dblValueOut.number()))) {
+           QString addZero = QString::number(dblValueOut.number()) + ".0";
+           m_pTxtValue1->setText(addZero);
+        } else {
+           m_pTxtValue1->setText(QString::number(dblValueOut.number(), 'g', 15));
         }
     }
 }
@@ -135,11 +180,14 @@ QGraphicsWidget *UnitConverter::graphicsWidget()
         Plasma::Label *pLabel = new Plasma::Label(this);
         pLabel->nativeWidget()->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
         pLabel->setText(i18n("Convert:"));
-        m_pCmbCategory = new Plasma::ComboBox(this);
+        m_pCmbCategory = new ComboBox(this);
+        connect(m_pCmbCategory, SIGNAL(mousePressed()), this, SLOT(raise()));
         m_pCmbCategory->setZValue(2);
 
-        m_pCmbUnit1 = new Plasma::ComboBox(this);
-        m_pCmbUnit2 = new Plasma::ComboBox(this);
+        m_pCmbUnit1 = new ComboBox(this);
+        m_pCmbUnit2 = new ComboBox(this);
+        connect(m_pCmbUnit1, SIGNAL(mousePressed()), this, SLOT(raise()));
+        connect(m_pCmbUnit2, SIGNAL(mousePressed()), this, SLOT(raise()));
         m_pCmbUnit1->setZValue(1);
         m_pCmbUnit2->setZValue(1);
         m_pTxtValue1 = new Plasma::LineEdit(this);
@@ -164,8 +212,10 @@ QGraphicsWidget *UnitConverter::graphicsWidget()
         }
         m_pCmbCategory->nativeWidget()->model()->sort(0);
 
-        connect(m_pTxtValue1->nativeWidget(), SIGNAL(textChanged(const QString&)),
-                this, SLOT(sltValueChanged(const QString&)));
+        connect(m_pTxtValue1->nativeWidget(), SIGNAL(textEdited(QString)),
+                this, SLOT(sltValueChanged(QString)));
+        connect(m_pTxtValue2->nativeWidget(), SIGNAL(textEdited(QString)),
+                this, SLOT(sltValueChangedReverse(QString)));
         connect(m_pCmbCategory->nativeWidget(), SIGNAL(currentIndexChanged(int)),
                 this, SLOT(sltCategoryChanged(int)));
         connect(m_pCmbUnit1->nativeWidget(), SIGNAL(currentIndexChanged(int)),
