@@ -1,11 +1,11 @@
 /*
  *   Copyright (C) 2007 Tobias Koenig <tokoe@kde.org>
- *   Copyright  2008 by Anne-Marie Mahfouf <annma@kde.org>                 
+ *   Copyright  2008 by Anne-Marie Mahfouf <annma@kde.org>
  *
- *   This program is free software; you can redistribute it and/or modify  
- *   it under the terms of the GNU General Public License as published by  
- *   the Free Software Foundation; either version 2 of the License, or     
- *   (at your option) any later version.   
+ *   This program is free software; you can redistribute it and/or modify
+ *   it under the terms of the GNU General Public License as published by
+ *   the Free Software Foundation; either version 2 of the License, or
+ *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,7 +20,6 @@
 
 #include "epodprovider.h"
 
-#include <QtCore/QDate>
 #include <QtCore/QRegExp>
 #include <QtGui/QImage>
 
@@ -43,7 +42,6 @@ class EpodProvider::Private
 
     EpodProvider *mParent;
     QByteArray mPage;
-    QDate mDate;
     QImage mImage;
 };
 
@@ -54,23 +52,18 @@ void EpodProvider::Private::pageRequestFinished(KJob *_job)
 	emit mParent->error( mParent );
 	return;
     }
-    
-    const QString data = QString::fromUtf8( job->data() );
-    //kDebug() << "####### data " << data << endl;
 
-    const QString pattern( "<strong><a href=*" );
+    const QString data = QString::fromUtf8( job->data() );
+
+    const QString pattern( QLatin1String( "http://epod.usra.edu/.a/*-pi" ) );
     QRegExp exp( pattern );
     exp.setPatternSyntax(QRegExp::Wildcard);
-    
-    int pos = exp.indexIn( data ) + pattern.length();
-    //kDebug() << "length " << pattern.length() << endl;
-    const QString sub = data.mid( pos, pattern.length()+41);
-    //kDebug() << "####### sub " << sub << endl; //TODO should return *.jpg
-    
-    KUrl url(  sub  );
 
-    KIO::StoredTransferJob *imageJob = KIO::storedGet( url );
-    QObject::connect(imageJob, SIGNAL( finished( KJob* )), mParent, SLOT( imageRequestFinished( KJob* ) ) );
+    int pos = exp.indexIn( data ) + pattern.length();
+    const QString sub = data.mid( pos-4, pattern.length()+6);
+    KUrl url( QString(QLatin1String( "http://epod.usra.edu/.a/%1-pi" )) .arg(sub)  );
+    KIO::StoredTransferJob *imageJob = KIO::storedGet( url, KIO::NoReload, KIO::HideProgressInfo );
+    QObject::connect(imageJob, SIGNAL(finished(KJob*)), mParent, SLOT(imageRequestFinished(KJob*)) );
 }
 
 void EpodProvider::Private::imageRequestFinished( KJob *_job)
@@ -81,6 +74,7 @@ void EpodProvider::Private::imageRequestFinished( KJob *_job)
 	return;
     }
 
+    // FIXME: this really should be done in a thread as this can block
     mImage = QImage::fromData( job->data() );
     emit mParent->finished( mParent );
 }
@@ -88,16 +82,10 @@ void EpodProvider::Private::imageRequestFinished( KJob *_job)
 EpodProvider::EpodProvider( QObject *parent, const QVariantList &args )
     : PotdProvider( parent, args ), d( new Private( this ) )
 {
-    const QString type = args[ 0 ].toString();
-    if ( type == "Date" )
-        d->mDate = args[ 1 ].toDate();
-    else
-	Q_ASSERT( false && "Invalid type passed to potd provider" );
+    KUrl url( QLatin1String( "http://epod.usra.edu/blog/" ) );
+    KIO::StoredTransferJob *job = KIO::storedGet( url, KIO::NoReload, KIO::HideProgressInfo );
 
-    KUrl url( QString( "http://epod.usra.edu/blog/" ) );
-    KIO::StoredTransferJob *job = KIO::storedGet( url );
-
-    connect( job, SIGNAL( finished( KJob* ) ), SLOT( pageRequestFinished( KJob* ) ) );
+    connect( job, SIGNAL(finished(KJob*)), SLOT(pageRequestFinished(KJob*)) );
 }
 
 EpodProvider::~EpodProvider()
@@ -110,9 +98,4 @@ QImage EpodProvider::image() const
     return d->mImage;
 }
 
-QString EpodProvider::identifier() const
-{
-    return QString( "epod:%1" ).arg( d->mDate.toString( Qt::ISODate ));
-}
-
 #include "epodprovider.moc"
