@@ -3,10 +3,10 @@
  *   Copyright  2008 by Anne-Marie Mahfouf <annma@kde.org>
  *   Copyright  2008 by Georges Toth <gtoth@trypill.org>
  *
- *   This program is free software; you can redistribute it and/or modify  
- *   it under the terms of the GNU General Public License as published by  
- *   the Free Software Foundation; either version 2 of the License, or     
- *   (at your option) any later version.   
+ *   This program is free software; you can redistribute it and/or modify
+ *   it under the terms of the GNU General Public License as published by
+ *   the Free Software Foundation; either version 2 of the License, or
+ *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -21,7 +21,6 @@
 
 #include "flickrprovider.h"
 
-#include <QtCore/QDate>
 #include <QtCore/QRegExp>
 #include <QtGui/QImage>
 #include <QXmlStreamReader>
@@ -46,7 +45,6 @@ class FlickrProvider::Private
 
     FlickrProvider *mParent;
     QByteArray mPage;
-    QDate mDate;
     QDate mActualDate;
     QImage mImage;
 
@@ -79,22 +77,22 @@ void FlickrProvider::Private::pageRequestFinished( KJob *_job )
         if (xml.isStartElement()) {
             if (xml.name() == "rsp") {
                 /* no pictures available for the specified parameters */
-                if (xml.attributes().value ( "stat" ).toString() == "fail") {
+                if (xml.attributes().value ( QLatin1String( "stat" ) ).toString() == QLatin1String( "fail" ) ) {
                     /* To be sure, decrement the date to two days earlier... @TODO */
                     mActualDate = mActualDate.addDays(-2);
 
-                            KUrl url( "http://api.flickr.com/services/rest/?api_key=a902f4e74cf1e7bce231742d8ffb46b4&method=flickr.interestingness.getList&date=" + mActualDate.toString( Qt::ISODate) );
+                            KUrl url( QLatin1String( "http://api.flickr.com/services/rest/?api_key=a902f4e74cf1e7bce231742d8ffb46b4&method=flickr.interestingness.getList&date=" ) + mActualDate.toString( Qt::ISODate) );
                             KIO::StoredTransferJob *pageJob = KIO::storedGet(url, KIO::NoReload, KIO::HideProgressInfo);
-                            mParent->connect( pageJob, SIGNAL( finished( KJob* ) ), SLOT( pageRequestFinished( KJob* ) ) );
+                            mParent->connect( pageJob, SIGNAL(finished(KJob*)), SLOT(pageRequestFinished(KJob*)) );
                     return;
                 }
-            } else if (xml.name() == "photo") {
-                if (xml.attributes().value ( "ispublic" ).toString() != "1")
+            } else if (xml.name() == QLatin1String( "photo" )) {
+                if (xml.attributes().value ( QLatin1String( "ispublic" ) ).toString() != QLatin1String( "1" ))
                 continue;
 
-                QString fileUrl = QString("http://farm" + xml.attributes().value ( "farm" ).toString() + ".static.flickr.com/"
-                + xml.attributes().value ( "server" ).toString() + '/' + xml.attributes().value ( "id" ).toString()
-                + '_' + xml.attributes().value ( "secret" ).toString() + ".jpg");
+                QString fileUrl = QString(QLatin1String( "http://farm" ) + xml.attributes().value ( QLatin1String( "farm" ) ).toString() + QLatin1String( ".static.flickr.com/" )
+                + xml.attributes().value ( QLatin1String( "server" ) ).toString() + QLatin1Char( '/' ) + xml.attributes().value ( QLatin1String( "id" ) ).toString()
+                                          + QLatin1Char( '_' ) + xml.attributes().value ( QLatin1String( "secret" ) ).toString() + QLatin1String( ".jpg" ));
 
                 m_photoList.append(fileUrl);
             }
@@ -108,7 +106,7 @@ void FlickrProvider::Private::pageRequestFinished( KJob *_job )
     if (m_photoList.begin() != m_photoList.end()) {
         KUrl url( m_photoList.at(qrand() % m_photoList.size()) );
             KIO::StoredTransferJob *imageJob = KIO::storedGet(url, KIO::NoReload, KIO::HideProgressInfo);
-            mParent->connect( imageJob, SIGNAL( finished( KJob* ) ), SLOT( imageRequestFinished( KJob* ) ) );
+            mParent->connect( imageJob, SIGNAL(finished(KJob*)), SLOT(imageRequestFinished(KJob*)) );
     } else {
         kDebug() << "empty list";
     }
@@ -129,17 +127,12 @@ void FlickrProvider::Private::imageRequestFinished( KJob *_job )
 FlickrProvider::FlickrProvider( QObject *parent, const QVariantList &args )
     : PotdProvider( parent, args ), d( new Private( this ) )
 {
-    const QString type = args[ 0 ].toString();
-    if ( type == "Date" ) {
-        d->mDate = args[ 1 ].toDate();
-        d->mActualDate = d->mDate;
-    } else {
-        Q_ASSERT( false && "Invalid type passed to potd provider" );
-    }
-
-    KUrl url("http://api.flickr.com/services/rest/?api_key=a902f4e74cf1e7bce231742d8ffb46b4&method=flickr.interestingness.getList&date=" + d->mDate.toString( Qt::ISODate ) );
-    KIO::StoredTransferJob *job = KIO::storedGet( url );
-    connect( job, SIGNAL( finished( KJob* ) ), SLOT( pageRequestFinished( KJob* ) ) );
+    d->mActualDate = date();
+    KUrl url(QLatin1String(
+                "http://api.flickr.com/services/rest/?api_key=a902f4e74cf1e7bce231742d8ffb46b4&method=flickr.interestingness.getList&date="
+                ) + date().toString( Qt::ISODate ) );
+    KIO::StoredTransferJob *job = KIO::storedGet( url, KIO::NoReload, KIO::HideProgressInfo );
+    connect( job, SIGNAL(finished(KJob*)), SLOT(pageRequestFinished(KJob*)) );
 }
 
 FlickrProvider::~FlickrProvider()
@@ -152,9 +145,4 @@ QImage FlickrProvider::image() const
     return d->mImage;
 }
 
-QString FlickrProvider::identifier() const
-{
-    return QString( "flickr:%1" ).arg( d->mDate.toString( Qt::ISODate ));
-}
-
 #include "flickrprovider.moc"
