@@ -25,7 +25,7 @@
 #include <KUnitConversion/Converter>
 #include <KUnitConversion/UnitCategory>
 
-#define CONVERSION_CHAR '>'
+#define CONVERSION_CHAR QLatin1Char( '>' )
 
 using namespace KUnitConversion;
 
@@ -62,7 +62,7 @@ public:
             if (type == GetString && number) {
                 break;
             }
-            if(current == CONVERSION_CHAR) {
+            if(current == QLatin1Char( CONVERSION_CHAR )) {
                 break;
             }
             ++m_index;
@@ -76,7 +76,7 @@ public:
         if (ch.isNumber()) {
             return true;
         }
-        if (QString(".,-+").contains(ch)) {
+        if (QString(QLatin1String( ".,-+" )).contains( ch )) {
             return true;
         }
         return false;
@@ -124,11 +124,11 @@ ConverterRunner::ConverterRunner(QObject* parent, const QVariantList &args)
     : Plasma::AbstractRunner(parent, args)
 {
     Q_UNUSED(args)
-    setObjectName("Converter");
+    setObjectName(QLatin1String( "Converter" ));
 
-    m_separators << QString(CONVERSION_CHAR);
+    m_separators << QString( CONVERSION_CHAR );
     m_separators << i18nc("list of words that can used as amount of 'unit1' [in|to|as] 'unit2'",
-                          "in;to;as").split(';');
+                          "in;to;as").split(QLatin1Char( ';' ));
 
     //can not ignore commands: we have things like m4
     setIgnoredTypes(Plasma::RunnerContext::Directory | Plasma::RunnerContext::File |
@@ -137,7 +137,7 @@ ConverterRunner::ConverterRunner(QObject* parent, const QVariantList &args)
     QString description = i18n("Converts the value of :q: when :q: is made up of "
                                "\"value unit [>, to, as, in] unit\". You can use the "
                                "Unit converter applet to find all available units.");
-    addSyntax(Plasma::RunnerSyntax("=:q:", description));
+    addSyntax(Plasma::RunnerSyntax(QLatin1String( "=:q:" ), description));
 }
 
 ConverterRunner::~ConverterRunner()
@@ -171,7 +171,7 @@ void ConverterRunner::match(Plasma::RunnerContext &context)
     const QString s = cmd.get(StringParser::GetString);
 
     if (!s.isEmpty() && !m_separators.contains(s)) {
-        unit1 += ' ' + s;
+        unit1 += QLatin1Char( ' ' ) + s;
     }
     if (s.isEmpty() || !m_separators.contains(s)) {
         cmd.pass(m_separators);
@@ -180,41 +180,72 @@ void ConverterRunner::match(Plasma::RunnerContext &context)
 
     Converter converter;
     UnitCategory* category = converter.categoryForUnit(unit1);
-    if (!category) {
-        return;
-    }
-    Value v = category->convert(Value(value, unit1), unit2);
-    if (v.isValid()) {
-        Plasma::QueryMatch match(this);
-        match.setType(Plasma::QueryMatch::InformationalMatch);
-        match.setIcon(KIcon("edit-copy"));
-        match.setText(v.toString());
-        match.setData(v.number());
-        context.addMatch(term, match);
-    } else if (!unit2.isEmpty()) {
-        const QStringList units = category->allUnits();
-        QSet<UnitPtr> matchingUnits;
-        foreach (const QString& s, units) {
-            if (s.startsWith(unit2, Qt::CaseInsensitive)) {
-                matchingUnits << category->unit(s);
+    bool found = false;
+    if (category->id() == InvalidCategory) {
+        foreach (category, converter.categories()) {
+            foreach (const QString& s, category->allUnits()) {
+                if (s.compare(unit1, Qt::CaseInsensitive) == 0) {
+                    found = true;
+                    break;
+                }
+            }
+            if (found) {
+                break;
             }
         }
-        foreach (const UnitPtr& u, matchingUnits) {
-            v = category->convert(Value(value, unit1), u->symbol());
-            Plasma::QueryMatch match(this);
-            match.setType(Plasma::QueryMatch::InformationalMatch);
-            match.setIcon(KIcon("edit-copy"));
-            match.setText(v.toString());
-            match.setData(v.number());
-            context.addMatch(term, match);
+        if (!found) {
+            return;
         }
     }
-    if (v.isValid() && !v.unit()->category()->description().isEmpty()) {
+
+    QList<UnitPtr> units;
+
+    if (!unit2.isEmpty()) {
+        UnitPtr u = category->unit(unit2);
+        if (!u.isNull() && u->isValid()) {
+            units.append(u);
+            config().writeEntry(category->name(), u->symbol());
+        } else {
+            const QStringList unitStrings = category->allUnits();
+            QSet<UnitPtr> matchingUnits;
+            foreach (const QString& s, unitStrings) {
+                if (s.startsWith(unit2, Qt::CaseInsensitive)) {
+                    matchingUnits << category->unit(s);
+                }
+            }
+            units = matchingUnits.toList();
+            if (units.count() == 1) {
+                config().writeEntry(category->name(), units[0]->symbol());
+            }
+        }
+    } else {
+        units = category->mostCommonUnits();
+        UnitPtr u = category->unit(config().readEntry(category->name()));
+        if (!u.isNull() && units.indexOf(u) < 0) {
+            units << u;
+        }
+    }
+
+    UnitPtr u1 = category->unit(unit1);
+    foreach (const UnitPtr& u, units) {
+        if (u1 == u) {
+            continue;
+        }
+        Value v = category->convert(Value(value.toDouble(), u1), u);
+        Plasma::QueryMatch match(this);
+        match.setType(Plasma::QueryMatch::InformationalMatch);
+        match.setIcon(KIcon(QLatin1String( "edit-copy" )));
+        match.setText(QString(QLatin1String( "%1 (%2)" )).arg(v.toString()).arg(u->symbol()));
+        match.setData(v.number());
+        context.addMatch(term, match);
+    }
+
+    if (units.count() > 0 && !category->description().isEmpty()) {
         Plasma::QueryMatch match(this);
         match.setType(Plasma::QueryMatch::PossibleMatch);
-        match.setIcon(KIcon("document-open-remote"));
-        match.setText(v.unit()->category()->description());
-        match.setData(v.unit()->category()->url().prettyUrl());
+        match.setIcon(KIcon(QLatin1String( "document-open-remote" )));
+        match.setText(category->description());
+        match.setData(category->url().prettyUrl());
         context.addMatch(term, match);
     }
 }
