@@ -15,9 +15,10 @@
 
 #include <QPainter>
 
+#include <KDebug>
 #include <KDirWatch>
 #include <KFileDialog>
-#include <KNS/Engine>
+#include <KNS3/DownloadDialog>
 #include <KStandardDirs>
 
 #include <Plasma/Theme>
@@ -39,7 +40,7 @@ Virus::Virus(QObject *parent, const QVariantList &args)
 {
     connect(this, SIGNAL(renderCompleted(QImage)), this, SLOT(updateBackground(QImage)));
     connect(&alife, SIGNAL(finished()), this, SLOT(virusUpdated()));
-    
+
     //m_timer = QTimer(this);
     m_timer.setSingleShot(true);
     connect(&m_timer, SIGNAL(timeout()), this, SLOT(requestUpdate()));
@@ -54,14 +55,14 @@ Virus::~Virus()
 void Virus::init(const KConfigGroup &config)
 {
     m_timer.stop();
-    m_mode = "SingleImage";//renderingMode().name();
+    m_mode = QLatin1String( "SingleImage" );//renderingMode().name();
     calculateGeometry();
 
     m_resizeMethod = (ResizeMethod)config.readEntry("wallpaperposition", (int)ScaledResize);
     m_wallpaper = config.readEntry("wallpaper", QString());
     if (m_wallpaper.isEmpty()) {
         m_wallpaper = Plasma::Theme::defaultTheme()->wallpaperPath();
-        int index = m_wallpaper.indexOf("/contents/images/");
+        int index = m_wallpaper.indexOf(QLatin1String( "/contents/images/" ));
         if (index > -1) { // We have file from package -> get path to package
             m_wallpaper = m_wallpaper.left(index);
         }
@@ -121,13 +122,20 @@ QWidget* Virus::createConfigurationInterface(QWidget* parent)
     m_uiVirus.setupUi(m_configWidget);
 
     qreal ratio = m_size.isEmpty() ? 1.0 : m_size.width() / qreal(m_size.height());
-    m_model = new BackgroundListModel(ratio, this, m_configWidget);
+    m_model = new BackgroundListModel(this, m_configWidget);
     m_model->setResizeMethod(m_resizeMethod);
     m_model->setWallpaperSize(m_size);
     m_model->reload(m_usersWallpapers);
     m_uiVirus.m_view->setModel(m_model);
     m_uiVirus.m_view->setItemDelegate(new BackgroundDelegate(m_uiVirus.m_view,
                                                                  ratio, m_configWidget));
+    
+    m_uiVirus.m_view->setMinimumWidth((BackgroundDelegate::SCREENSHOT_SIZE + BackgroundDelegate::MARGIN * 2 +
+                                           BackgroundDelegate::BLUR_INCREMENT) * 3 +
+                                           m_uiVirus.m_view->spacing() * 4 +
+                                           QApplication::style()->pixelMetric(QStyle::PM_ScrollBarExtent) +
+                                           QApplication::style()->pixelMetric(QStyle::PM_DefaultFrameWidth) * 2 + 7);
+    
     m_uiVirus.m_view->setVerticalScrollMode(QAbstractItemView::ScrollPerPixel);
     QModelIndex index = m_model->indexOf(m_wallpaper);
     if (index.isValid()) {
@@ -137,9 +145,9 @@ QWidget* Virus::createConfigurationInterface(QWidget* parent)
             fillMetaInfo(b);
         }
     }
-    connect(m_uiVirus.m_view, SIGNAL(activated(const QModelIndex &)), this, SLOT(pictureChanged(const QModelIndex &)));
+    connect(m_uiVirus.m_view, SIGNAL(activated(QModelIndex)), this, SLOT(pictureChanged(QModelIndex)));
 
-    m_uiVirus.m_pictureUrlButton->setIcon(KIcon("document-open"));
+    m_uiVirus.m_pictureUrlButton->setIcon(KIcon(QLatin1String( "document-open" )));
     connect(m_uiVirus.m_pictureUrlButton, SIGNAL(clicked()), this, SLOT(showFileDialog()));
 
     m_uiVirus.m_resizeMethod->addItem(i18n("Scaled & Cropped"), ScaledAndCroppedResize);
@@ -156,10 +164,10 @@ QWidget* Virus::createConfigurationInterface(QWidget* parent)
     }
     connect(m_uiVirus.m_resizeMethod, SIGNAL(currentIndexChanged(int)),
             this, SLOT(positioningChanged(int)));
-    m_uiVirus.m_newStuff->setIcon(KIcon("get-hot-new-stuff"));
-   
+    m_uiVirus.m_newStuff->setIcon(KIcon(QLatin1String( "get-hot-new-stuff" )));
+
     m_uiVirus.m_color->setColor(m_color);
-    connect(m_uiVirus.m_color, SIGNAL(changed(const QColor&)), this, SLOT(colorChanged(const QColor&)));
+    connect(m_uiVirus.m_color, SIGNAL(changed(QColor)), this, SLOT(colorChanged(QColor)));
 
     m_uiVirus.m_maxCells->setValue(alife.getMaxViruses());
     connect(m_uiVirus.m_maxCells, SIGNAL(valueChanged(int)), this, SLOT(maxCellsChanged(int)));
@@ -171,11 +179,11 @@ QWidget* Virus::createConfigurationInterface(QWidget* parent)
     connect(m_uiVirus.m_showCells, SIGNAL(stateChanged(int)), this, SLOT(showCellsChanged(int)));
 
     connect(m_uiVirus.m_newStuff, SIGNAL(clicked()), this, SLOT(getNewWallpaper()));
-    
+
     connect(this, SIGNAL(settingsChanged(bool)), parent, SLOT(settingsChanged(bool)));
-    
-    connect(m_uiVirus.m_view, SIGNAL(clicked(const QModelIndex &)), this, SLOT(modified()));
-    
+
+    connect(m_uiVirus.m_view, SIGNAL(clicked(QModelIndex)), this, SLOT(modified()));
+
     return m_configWidget;
 }
 
@@ -232,14 +240,29 @@ void Virus::setSingleImage()
     }
 
     QString img;
-    Plasma::Package b(m_wallpaper, packageStructure(this));
 
-    img = b.filePath("preferred");
-    kDebug() << img << m_wallpaper;
-    if (img.isEmpty()) {
-        img = m_wallpaper;
+    if (QDir::isAbsolutePath(m_wallpaper)) {
+        Plasma::Package b(m_wallpaper, packageStructure(this));
+        img = b.filePath("preferred");
+        kDebug() << img << m_wallpaper;
+
+        if (img.isEmpty()) {
+            img = m_wallpaper;
+        }
+    //if it's not an absolute path, check if it's just a wallpaper name
+    } else {
+        const QString path = KStandardDirs::locate("wallpaper", m_wallpaper + QLatin1String( "/metadata.desktop" ));
+
+        if (!path.isEmpty()) {
+            QDir dir(path);
+            dir.cdUp();
+
+            Plasma::Package b(dir.path(), packageStructure(this));
+            img = b.filePath("preferred");
+        }
     }
 
+
     if (!m_size.isEmpty()) {
         renderWallpaper(img);
     }
@@ -247,13 +270,10 @@ void Virus::setSingleImage()
 
 void Virus::getNewWallpaper()
 {
-    KNS::Engine engine(m_configWidget);
-    if (engine.init("virus_wallpaper.knsrc")) {
-        KNS::Entry::List entries = engine.downloadDialogModal(m_configWidget);
-
-        if (entries.size() > 0 && m_model) {
-            m_model->reload();
-        }
+    KNS3::DownloadDialog dialog("virus_wallpaper.knsrc", m_configWidget);
+    dialog.exec();
+    if (dialog.changedEntries().size() > 0 && m_model) {
+        m_model->reload();
     }
 }
 
@@ -276,7 +296,7 @@ void Virus::pictureChanged(QModelIndex index)
     }
 
     fillMetaInfo(b);
-    if (b->structure()->contentsPrefix().isEmpty()) {
+    if (b->structure()->contentsPrefixPaths().isEmpty()) {
         // it's not a full package, but a single paper
         m_wallpaper = b->filePath("preferred");
     } else {
@@ -343,7 +363,7 @@ bool Virus::setMetadata(QLabel *label, const QString &text)
 void Virus::showFileDialog()
 {
     if (!m_dialog) {
-        m_dialog = new KFileDialog(KUrl(), "*.png *.jpeg *.jpg *.xcf *.svg *.svgz *.bmp", m_configWidget);
+        m_dialog = new KFileDialog(KUrl(), QLatin1String( "*.png *.jpeg *.jpg *.xcf *.svg *.svgz *.bmp" ), m_configWidget);
         m_dialog->setOperationMode(KFileDialog::Opening);
         m_dialog->setInlinePreviewShown(true);
         m_dialog->setCaption(i18n("Select Wallpaper Image File"));
@@ -404,7 +424,7 @@ void Virus::renderWallpaper(const QString& image)
 QString Virus::cacheId() const
 {
     QSize s = boundingRect().size().toSize();
-    return QString("%5_%3_%4_%1x%2").arg(s.width()).arg(s.height()).arg(m_color.name()).arg(m_resizeMethod).arg(m_img);
+    return QString(QLatin1String( "%5_%3_%4_%1x%2" )).arg(s.width()).arg(s.height()).arg(m_color.name()).arg(m_resizeMethod).arg(m_img);
 }
 
 void Virus::updateBackground(const QImage &img)
@@ -417,6 +437,7 @@ void Virus::updateBackground(const QImage &img)
 
 void Virus::suspendStartup(bool suspend)
 {
+    Q_UNUSED(suspend);
     //TODO: find out how to compile with that ksmserver
     /*if (m_startupResumed) {
         return;
